package sun.management.jmxremote;

import java.net.Socket;
import javax.net.ssl.SSLSocket;
import javax.net.ssl.SSLSocketFactory;
import java.net.ServerSocket;
import java.util.Iterator;
import java.util.Set;
import java.util.HashSet;
import java.security.Principal;
import com.sun.jmx.remote.security.JMXPluggableAuthenticator;
import javax.security.auth.Subject;
import java.util.Map;
import javax.management.remote.JMXAuthenticator;
import java.rmi.RemoteException;
import sun.rmi.server.UnicastServerRef;
import sun.rmi.server.UnicastServerRef2;
import java.rmi.server.RMIServerSocketFactory;
import java.rmi.server.RMIClientSocketFactory;
import java.rmi.Remote;
import com.sun.jmx.remote.internal.RMIExporter;
import java.net.MalformedURLException;
import java.rmi.server.RemoteObject;
import sun.rmi.server.UnicastRef;
import javax.rmi.ssl.SslRMIClientSocketFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManagerFactory;
import javax.net.ssl.KeyManagerFactory;
import java.security.KeyStore;
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import javax.rmi.ssl.SslRMIServerSocketFactory;
import java.io.IOException;
import sun.management.FileSystem;
import java.io.File;
import javax.management.remote.JMXConnectorServerFactory;
import java.net.UnknownHostException;
import java.net.InetAddress;
import javax.management.remote.JMXServiceURL;
import javax.management.MBeanServer;
import sun.management.ConnectorAddressLink;
import java.util.HashMap;
import java.lang.management.ManagementFactory;
import java.util.StringTokenizer;
import sun.management.AgentConfigurationError;
import java.util.Properties;
import sun.management.Agent;
import javax.management.remote.JMXConnectorServer;
import java.rmi.NoSuchObjectException;
import java.rmi.server.UnicastRemoteObject;
import com.sun.jmx.remote.util.ClassLogger;
import java.rmi.registry.Registry;

public final class ConnectorBootstrap
{
    private static Registry registry;
    private static final ClassLogger log;
    
    public static void unexportRegistry() {
        try {
            if (ConnectorBootstrap.registry != null) {
                UnicastRemoteObject.unexportObject(ConnectorBootstrap.registry, true);
                ConnectorBootstrap.registry = null;
            }
        }
        catch (NoSuchObjectException ex) {}
    }
    
    public static synchronized JMXConnectorServer initialize() {
        Properties loadManagementProperties = Agent.loadManagementProperties();
        if (loadManagementProperties == null) {
            return null;
        }
        return startRemoteConnectorServer(loadManagementProperties.getProperty("com.sun.management.jmxremote.port"), loadManagementProperties);
    }
    
    public static synchronized JMXConnectorServer initialize(String s, Properties properties) {
        return startRemoteConnectorServer(s, properties);
    }
    
    public static synchronized JMXConnectorServer startRemoteConnectorServer(String s, Properties properties) {
        int int1;
        try {
            int1 = Integer.parseInt(s);
        }
        catch (NumberFormatException ex) {
            throw new AgentConfigurationError("agent.err.invalid.jmxremote.port", ex, new String[] { s });
        }
        if (int1 < 0) {
            throw new AgentConfigurationError("agent.err.invalid.jmxremote.port", new String[] { s });
        }
        int int2 = 0;
        String property = properties.getProperty("com.sun.management.jmxremote.rmi.port");
        try {
            if (property != null) {
                int2 = Integer.parseInt(property);
            }
        }
        catch (NumberFormatException ex2) {
            throw new AgentConfigurationError("agent.err.invalid.jmxremote.rmi.port", ex2, new String[] { property });
        }
        if (int2 < 0) {
            throw new AgentConfigurationError("agent.err.invalid.jmxremote.rmi.port", new String[] { property });
        }
        String property2 = properties.getProperty("com.sun.management.jmxremote.authenticate", "true");
        int booleanValue = (boolean)Boolean.valueOf(property2);
        String property3 = properties.getProperty("com.sun.management.jmxremote.ssl", "true");
        int booleanValue2 = (boolean)Boolean.valueOf(property3);
        String property4 = properties.getProperty("com.sun.management.jmxremote.registry.ssl", "false");
        int booleanValue3 = (boolean)Boolean.valueOf(property4);
        String property5 = properties.getProperty("com.sun.management.jmxremote.ssl.enabled.cipher.suites");
        String[] array = null;
        if (property5 != null) {
            StringTokenizer o = new StringTokenizer(property5, ",");
            int countTokens = o.countTokens();
            array = new String[countTokens];
            for (int i = 0; i < countTokens; ++i) {
                array[i] = o.nextToken();
            }
        }
        String property6 = properties.getProperty("com.sun.management.jmxremote.ssl.enabled.protocols");
        String[] array2 = null;
        if (property6 != null) {
            StringTokenizer o2 = new StringTokenizer(property6, ",");
            int countTokens2 = o2.countTokens();
            array2 = new String[countTokens2];
            for (int j = 0; j < countTokens2; ++j) {
                array2[j] = o2.nextToken();
            }
        }
        String property7 = properties.getProperty("com.sun.management.jmxremote.ssl.need.client.auth", "false");
        int booleanValue4 = (boolean)Boolean.valueOf(property7);
        String property8 = properties.getProperty("com.sun.management.jmxremote.ssl.config.file");
        String property9 = null;
        String property10 = null;
        String property11 = null;
        if (!(booleanValue == 0)) {
            property9 = properties.getProperty("com.sun.management.jmxremote.login.config");
            if (property9 == null) {
                property10 = properties.getProperty("com.sun.management.jmxremote.password.file", getDefaultFileName("jmxremote.password"));
                checkPasswordFile(property10);
            }
            property11 = properties.getProperty("com.sun.management.jmxremote.access.file", getDefaultFileName("jmxremote.access"));
            checkAccessFile(property11);
        }
        String property12 = properties.getProperty("com.sun.management.jmxremote.host");
        if (ConnectorBootstrap.log.debugOn()) {
            ConnectorBootstrap.log.debug("startRemoteConnectorServer", Agent.getText("jmxremote.ConnectorBootstrap.starting") + "\n\t" + "com.sun.management.jmxremote.port" + "=" + int1 + ((property12 == null) ? "" : ("\n\tcom.sun.management.jmxremote.host=" + property12)) + "\n\t" + "com.sun.management.jmxremote.rmi.port" + "=" + int2 + "\n\t" + "com.sun.management.jmxremote.ssl" + "=" + (boolean)booleanValue2 + "\n\t" + "com.sun.management.jmxremote.registry.ssl" + "=" + (boolean)booleanValue3 + "\n\t" + "com.sun.management.jmxremote.ssl.config.file" + "=" + property8 + "\n\t" + "com.sun.management.jmxremote.ssl.enabled.cipher.suites" + "=" + property5 + "\n\t" + "com.sun.management.jmxremote.ssl.enabled.protocols" + "=" + property6 + "\n\t" + "com.sun.management.jmxremote.ssl.need.client.auth" + "=" + (boolean)booleanValue4 + "\n\t" + "com.sun.management.jmxremote.authenticate" + "=" + (boolean)booleanValue + (booleanValue ? ((property9 == null) ? ("\n\tcom.sun.management.jmxremote.password.file=" + property10) : ("\n\tcom.sun.management.jmxremote.login.config=" + property9)) : ("\n\t" + Agent.getText("jmxremote.ConnectorBootstrap.noAuthentication"))) + (booleanValue ? ("\n\tcom.sun.management.jmxremote.access.file=" + property11) : "") + "");
        }
        MBeanServer platformMBeanServer = ManagementFactory.getPlatformMBeanServer();
        JMXConnectorServer jmxConnectorServer;
        JMXServiceURL jmxRemoteURL;
        try {
            ConnectorBootstrap.JMXConnectorServerData exportMBeanServer = exportMBeanServer(platformMBeanServer, int1, int2, (boolean)booleanValue2, (boolean)booleanValue3, property8, array, array2, (boolean)booleanValue4, (boolean)booleanValue, property9, property10, property11, property12);
            jmxConnectorServer = exportMBeanServer.jmxConnectorServer;
            jmxRemoteURL = exportMBeanServer.jmxRemoteURL;
            ConnectorBootstrap.log.config("startRemoteConnectorServer", Agent.getText("jmxremote.ConnectorBootstrap.ready", new String[] { jmxRemoteURL.toString() }));
        }
        catch (Exception ex3) {
            throw new AgentConfigurationError("agent.err.exception", ex3, new String[] { ex3.toString() });
        }
        try {
            HashMap o3 = new HashMap();
            o3.put("remoteAddress", jmxRemoteURL.toString());
            o3.put("authenticate", property2);
            o3.put("ssl", property3);
            o3.put("sslRegistry", property4);
            o3.put("sslNeedClientAuth", property7);
            ConnectorAddressLink.exportRemote(o3);
        }
        catch (Exception t) {
            ConnectorBootstrap.log.debug("startRemoteConnectorServer", t);
        }
        return jmxConnectorServer;
    }
    
    public static JMXConnectorServer startLocalConnectorServer() {
        System.setProperty("java.rmi.server.randomIDs", "true");
        HashMap<String, Object> environment = new HashMap();
        environment.put("com.sun.jmx.remote.rmi.exporter", new ConnectorBootstrap.PermanentExporter());
        environment.put("jmx.remote.rmi.server.credential.types", new String[] { String[].class.getName(), String.class.getName() });
        String hostAddress = "localhost";
        InetAddress byName = null;
        try {
            byName = InetAddress.getByName(hostAddress);
            hostAddress = byName.getHostAddress();
        }
        catch (UnknownHostException ex) {}
        if (byName == null || !byName.isLoopbackAddress()) {
            hostAddress = "127.0.0.1";
        }
        MBeanServer platformMBeanServer = ManagementFactory.getPlatformMBeanServer();
        try {
            JMXServiceURL serviceURL = new JMXServiceURL("rmi", hostAddress, 0);
            Properties managementProperties = Agent.getManagementProperties();
            if (managementProperties == null) {
                managementProperties = new Properties();
            }
            if ((boolean)Boolean.valueOf(managementProperties.getProperty("com.sun.management.jmxremote.local.only", "true"))) {
                environment.put("jmx.remote.rmi.server.socket.factory", new LocalRMIServerSocketFactory());
            }
            JMXConnectorServer jmxConnectorServer = JMXConnectorServerFactory.newJMXConnectorServer(serviceURL, environment, platformMBeanServer);
            jmxConnectorServer.start();
            return jmxConnectorServer;
        }
        catch (Exception ex2) {
            throw new AgentConfigurationError("agent.err.exception", ex2, new String[] { ex2.toString() });
        }
    }
    
    private static void checkPasswordFile(String pathname) {
        if (pathname == null || pathname.length() == 0) {
            throw new AgentConfigurationError("agent.err.password.file.notset");
        }
        File file = new File(pathname);
        if (!file.exists()) {
            throw new AgentConfigurationError("agent.err.password.file.notfound", new String[] { pathname });
        }
        if (!file.canRead()) {
            throw new AgentConfigurationError("agent.err.password.file.not.readable", new String[] { pathname });
        }
        FileSystem open = FileSystem.open();
        try {
            if (open.supportsFileSecurity(file) && !open.isAccessUserOnly(file)) {
                ConnectorBootstrap.log.config("startRemoteConnectorServer", Agent.getText("jmxremote.ConnectorBootstrap.password.readonly", new String[] { pathname }));
                throw new AgentConfigurationError("agent.err.password.file.access.notrestricted", new String[] { pathname });
            }
        }
        catch (IOException ex) {
            throw new AgentConfigurationError("agent.err.password.file.read.failed", ex, new String[] { pathname });
        }
        return;
    }
    
    private static void checkAccessFile(String pathname) {
        if (pathname == null || pathname.length() == 0) {
            throw new AgentConfigurationError("agent.err.access.file.notset");
        }
        File file = new File(pathname);
        if (!file.exists()) {
            throw new AgentConfigurationError("agent.err.access.file.notfound", new String[] { pathname });
        }
        if (!file.canRead()) {
            throw new AgentConfigurationError("agent.err.access.file.not.readable", new String[] { pathname });
        }
        return;
    }
    
    private static void checkRestrictedFile(String pathname) {
        if (pathname == null || pathname.length() == 0) {
            throw new AgentConfigurationError("agent.err.file.not.set");
        }
        File file = new File(pathname);
        if (!file.exists()) {
            throw new AgentConfigurationError("agent.err.file.not.found", new String[] { pathname });
        }
        if (!file.canRead()) {
            throw new AgentConfigurationError("agent.err.file.not.readable", new String[] { pathname });
        }
        FileSystem open = FileSystem.open();
        try {
            if (open.supportsFileSecurity(file) && !open.isAccessUserOnly(file)) {
                ConnectorBootstrap.log.config("startRemoteConnectorServer", Agent.getText("jmxremote.ConnectorBootstrap.file.readonly", new String[] { pathname }));
                throw new AgentConfigurationError("agent.err.file.access.not.restricted", new String[] { pathname });
            }
        }
        catch (IOException ex) {
            throw new AgentConfigurationError("agent.err.file.read.failed", ex, new String[] { pathname });
        }
        return;
    }
    
    private static String getDefaultFileName(String str) {
        String separator = File.separator;
        return System.getProperty("java.home") + separator + "lib" + separator + "management" + separator + str;
    }
    
    private static SslRMIServerSocketFactory createSslRMIServerSocketFactory(String name, String[] array, String[] array2, boolean b, String s) {
        if (name == null) {
            return new ConnectorBootstrap.HostAwareSslSocketFactory(array, array2, b, s);
        }
        checkRestrictedFile(name);
        try {
            Properties properties = new Properties();
            try (final FileInputStream in = new FileInputStream(name)) {
                properties.load(new BufferedInputStream(in));
            }
            String property = properties.getProperty("javax.net.ssl.keyStore");
            String property2 = properties.getProperty("javax.net.ssl.keyStorePassword", "");
            String property3 = properties.getProperty("javax.net.ssl.trustStore");
            String property4 = properties.getProperty("javax.net.ssl.trustStorePassword", "");
            char[] charArray = null;
            if (property2.length() != 0) {
                charArray = property2.toCharArray();
            }
            char[] charArray2 = null;
            if (property4.length() != 0) {
                charArray2 = property4.toCharArray();
            }
            KeyStore instance = null;
            if (property != null) {
                instance = KeyStore.getInstance(KeyStore.getDefaultType());
                try (final Object stream = new FileInputStream(property)) {
                    instance.load(stream, charArray);
                }
            }
            KeyManagerFactory instance2 = KeyManagerFactory.getInstance(KeyManagerFactory.getDefaultAlgorithm());
            instance2.init(instance, charArray);
            KeyStore instance3 = null;
            if (property3 != null) {
                instance3 = KeyStore.getInstance(KeyStore.getDefaultType());
                try (final Object stream2 = new FileInputStream(property3)) {
                    instance3.load(stream2, charArray2);
                }
            }
            TrustManagerFactory instance4 = TrustManagerFactory.getInstance(TrustManagerFactory.getDefaultAlgorithm());
            instance4.init(instance3);
            SSLContext instance5 = SSLContext.getInstance("SSL");
            instance5.init(instance2.getKeyManagers(), instance4.getTrustManagers(), null);
            return new ConnectorBootstrap.HostAwareSslSocketFactory(instance5, array, array2, b, s);
        }
        catch (Exception ex) {
            throw new AgentConfigurationError("agent.err.exception", ex, new String[] { ex.toString() });
        }
    }
    
    private static ConnectorBootstrap.JMXConnectorServerData exportMBeanServer(MBeanServer mbeanServer, int n, int port, boolean b, boolean b2, String s, String[] array, String[] array2, boolean b3, boolean b4, String s2, String s3, String s4, String host) throws IOException, MalformedURLException {
        System.setProperty("java.rmi.server.randomIDs", "true");
        JMXServiceURL serviceURL = new JMXServiceURL("rmi", host, port);
        HashMap<String, Object> environment = new HashMap();
        ConnectorBootstrap.PermanentExporter permanentExporter = new ConnectorBootstrap.PermanentExporter();
        environment.put("com.sun.jmx.remote.rmi.exporter", permanentExporter);
        environment.put("jmx.remote.rmi.server.credential.types", new String[] { String[].class.getName(), String.class.getName() });
        int n2 = host != null && !b;
        if (b4) {
            if (s2 != null) {
                environment.put("jmx.remote.x.login.config", s2);
            }
            if (s3 != null) {
                environment.put("jmx.remote.x.password.file", s3);
            }
            environment.put("jmx.remote.x.access.file", s4);
            if (environment.get("jmx.remote.x.password.file") != null || environment.get("jmx.remote.x.login.config") != null) {
                environment.put("jmx.remote.authenticator", new ConnectorBootstrap.AccessFileCheckerAuthenticator(environment));
            }
        }
        SslRMIClientSocketFactory sslRMIClientSocketFactory = null;
        Object sslRMIServerSocketFactory = null;
        if (b || b2) {
            sslRMIClientSocketFactory = new SslRMIClientSocketFactory();
            sslRMIServerSocketFactory = createSslRMIServerSocketFactory(s, array, array2, b3, host);
        }
        if (b) {
            environment.put("jmx.remote.rmi.client.socket.factory", sslRMIClientSocketFactory);
            environment.put("jmx.remote.rmi.server.socket.factory", sslRMIServerSocketFactory);
        }
        if (!(n2 == 0)) {
            sslRMIServerSocketFactory = new HostAwareSocketFactory(host);
            environment.put("jmx.remote.rmi.server.socket.factory", sslRMIServerSocketFactory);
        }
        JMXConnectorServer jmxConnectorServer = null;
        try {
            jmxConnectorServer = JMXConnectorServerFactory.newJMXConnectorServer(serviceURL, environment, mbeanServer);
            jmxConnectorServer.start();
        }
        catch (IOException ex) {
            if (jmxConnectorServer == null || jmxConnectorServer.getAddress() == null) {
                throw new AgentConfigurationError("agent.err.connector.server.io.error", ex, new String[] { serviceURL.toString() });
            }
            throw new AgentConfigurationError("agent.err.connector.server.io.error", ex, new String[] { jmxConnectorServer.getAddress().toString() });
        }
        if (b2) {
            ConnectorBootstrap.registry = new SingleEntryRegistry(n, sslRMIClientSocketFactory, sslRMIServerSocketFactory, "jmxrmi", permanentExporter.firstExported);
        }
        else if (!(n2 == 0)) {
            ConnectorBootstrap.registry = new SingleEntryRegistry(n, sslRMIClientSocketFactory, sslRMIServerSocketFactory, "jmxrmi", permanentExporter.firstExported);
        }
        else {
            ConnectorBootstrap.registry = new SingleEntryRegistry(n, "jmxrmi", permanentExporter.firstExported);
        }
        return new ConnectorBootstrap.JMXConnectorServerData(jmxConnectorServer, new JMXServiceURL(String.format("service:jmx:rmi:///jndi/rmi://%s:%d/jmxrmi", new Object[] { serviceURL.getHost(), Integer.valueOf(((UnicastRef)((RemoteObject)ConnectorBootstrap.registry).getRef()).getLiveRef().getPort()) })));
    }
    
    private ConnectorBootstrap() {
    }
    
    static {
        ConnectorBootstrap.registry = null;
        log = new ClassLogger(ConnectorBootstrap.class.getPackage().getName(), "ConnectorBootstrap");
    }
    
    private static class JMXConnectorServerData
    {
        JMXConnectorServer jmxConnectorServer;
        JMXServiceURL jmxRemoteURL;
        
        public JMXConnectorServerData(JMXConnectorServer jmxConnectorServer, JMXServiceURL jmxRemoteURL) {
            this.jmxConnectorServer = jmxConnectorServer;
            this.jmxRemoteURL = jmxRemoteURL;
        }
    }
    
    private static class PermanentExporter implements RMIExporter
    {
        Remote firstExported;
        
        public Remote exportObject(Remote remote, int n, RMIClientSocketFactory csf, RMIServerSocketFactory ssf) throws RemoteException {
            synchronized (this) {
                if (this.firstExported == null) {
                    this.firstExported = remote;
                }
            }
            UnicastServerRef unicastServerRef;
            if (csf == null && ssf == null) {
                unicastServerRef = new UnicastServerRef(n);
            }
            else {
                unicastServerRef = new UnicastServerRef2(n, csf, ssf);
            }
            return unicastServerRef.exportObject(remote, null, true);
        }
        
        public boolean unexportObject(Remote obj, boolean force) throws NoSuchObjectException {
            return UnicastRemoteObject.unexportObject(obj, force);
        }
    }
    
    private static class AccessFileCheckerAuthenticator implements JMXAuthenticator
    {
        private final Map<String, Object> environment;
        private final Properties properties;
        private final String accessFile;
        
        public AccessFileCheckerAuthenticator(Map<String, Object> environment) throws IOException {
            this.environment = environment;
            this.accessFile = (String)environment.get("jmx.remote.x.access.file");
            this.properties = propertiesFromFile(this.accessFile);
        }
        
        @Override
        public Subject authenticate(Object o) {
            Subject authenticate = new JMXPluggableAuthenticator(this.environment).authenticate(o);
            this.checkAccessFileEntries(authenticate);
            return authenticate;
        }
        
        private void checkAccessFileEntries(Subject subject) {
            if (subject == null) {
                throw new SecurityException("Access denied! No matching entries found in the access file [" + this.accessFile + "] as the authenticated Subject is null");
            }
            Set<Principal> principals = subject.getPrincipals();
            Iterator<Object> iterator = principals.iterator();
            while (iterator.hasNext()) {
                if (this.properties.containsKey(((Principal)iterator.next()).getName())) {
                    return;
                }
                continue;
            }
            HashSet obj = new HashSet();
            Iterator<Object> iterator2 = principals.iterator();
            while (iterator2.hasNext()) {
                obj.add(((Principal)iterator2.next()).getName());
            }
            throw new SecurityException("Access denied! No entries found in the access file [" + this.accessFile + "] for any of the authenticated identities " + obj);
        }
        
        private static Properties propertiesFromFile(String name) throws IOException {
            Properties properties = new Properties();
            if (name == null) {
                return properties;
            }
            try (final FileInputStream inStream = new FileInputStream(name)) {
                properties.load(inStream);
            }
            return properties;
        }
    }
    
    private static class HostAwareSocketFactory implements RMIServerSocketFactory
    {
        private final String bindAddress;
        
        private HostAwareSocketFactory(String bindAddress) {
            this.bindAddress = bindAddress;
        }
        
        @Override
        public ServerSocket createServerSocket(int port) throws IOException {
            if (this.bindAddress == null) {
                return new ServerSocket(port);
            }
            try {
                return new ServerSocket(port, 0, InetAddress.getByName(this.bindAddress));
            }
            catch (UnknownHostException ex) {
                return new ServerSocket(port);
            }
        }
    }
    
    private static class HostAwareSslSocketFactory extends SslRMIServerSocketFactory
    {
        private final String bindAddress;
        private final String[] enabledCipherSuites;
        private final String[] enabledProtocols;
        private final boolean needClientAuth;
        private final SSLContext context;
        
        private HostAwareSslSocketFactory(String[] array, String[] array2, boolean b, String s) throws IllegalArgumentException {
            this(null, array, array2, b, s);
        }
        
        private HostAwareSslSocketFactory(SSLContext context, String[] enabledCipherSuites, String[] enabledProtocols, boolean needClientAuth, String bindAddress) throws IllegalArgumentException {
            this.context = context;
            this.bindAddress = bindAddress;
            this.enabledProtocols = enabledProtocols;
            this.enabledCipherSuites = enabledCipherSuites;
            this.needClientAuth = needClientAuth;
            checkValues(context, enabledCipherSuites, enabledProtocols);
        }
        
        @Override
        public ServerSocket createServerSocket(int n) throws IOException {
            if (this.bindAddress != null) {
                try {
                    return new ConnectorBootstrap.SslServerSocket(n, 0, InetAddress.getByName(this.bindAddress), this.context, this.enabledCipherSuites, this.enabledProtocols, this.needClientAuth);
                }
                catch (UnknownHostException ex) {
                    return new ConnectorBootstrap.SslServerSocket(n, this.context, this.enabledCipherSuites, this.enabledProtocols, this.needClientAuth);
                }
            }
            else {
                return new ConnectorBootstrap.SslServerSocket(n, this.context, this.enabledCipherSuites, this.enabledProtocols, this.needClientAuth);
            }
        }
        
        private static void checkValues(SSLContext sslContext, String[] enabledCipherSuites, String[] enabledProtocols) throws IllegalArgumentException {
            SSLSocketFactory sslSocketFactory = (sslContext == null) ? ((SSLSocketFactory)SSLSocketFactory.getDefault()) : sslContext.getSocketFactory();
            SSLSocket sslSocket = null;
            if (enabledCipherSuites != null || enabledProtocols != null) {
                try {
                    sslSocket = (SSLSocket)sslSocketFactory.createSocket();
                }
                catch (Exception cause) {
                    throw (IllegalArgumentException)new IllegalArgumentException("Unable to check if the cipher suites and protocols to enable are supported").initCause(cause);
                }
            }
            if (enabledCipherSuites != null) {
                sslSocket.setEnabledCipherSuites(enabledCipherSuites);
            }
            if (enabledProtocols != null) {
                sslSocket.setEnabledProtocols(enabledProtocols);
            }
        }
    }
    
    private static class SslServerSocket extends ServerSocket
    {
        private static SSLSocketFactory defaultSSLSocketFactory;
        private final String[] enabledCipherSuites;
        private final String[] enabledProtocols;
        private final boolean needClientAuth;
        private final SSLContext context;
        
        private SslServerSocket(int port, SSLContext context, String[] enabledCipherSuites, String[] enabledProtocols, boolean needClientAuth) throws IOException {
            super(port);
            this.enabledProtocols = enabledProtocols;
            this.enabledCipherSuites = enabledCipherSuites;
            this.needClientAuth = needClientAuth;
            this.context = context;
        }
        
        private SslServerSocket(int port, int backlog, InetAddress bindAddr, SSLContext context, String[] enabledCipherSuites, String[] enabledProtocols, boolean needClientAuth) throws IOException {
            super(port, backlog, bindAddr);
            this.enabledProtocols = enabledProtocols;
            this.enabledCipherSuites = enabledCipherSuites;
            this.needClientAuth = needClientAuth;
            this.context = context;
        }
        
        @Override
        public Socket accept() throws IOException {
            SSLSocketFactory sslSocketFactory = (this.context == null) ? getDefaultSSLSocketFactory() : this.context.getSocketFactory();
            Socket accept = super.accept();
            SSLSocket sslSocket = (SSLSocket)sslSocketFactory.createSocket(accept, accept.getInetAddress().getHostName(), accept.getPort(), true);
            sslSocket.setUseClientMode(false);
            if (this.enabledCipherSuites != null) {
                sslSocket.setEnabledCipherSuites(this.enabledCipherSuites);
            }
            if (this.enabledProtocols != null) {
                sslSocket.setEnabledProtocols(this.enabledProtocols);
            }
            sslSocket.setNeedClientAuth(this.needClientAuth);
            return sslSocket;
        }
        
        private static synchronized SSLSocketFactory getDefaultSSLSocketFactory() {
            if (ConnectorBootstrap.SslServerSocket.defaultSSLSocketFactory == null) {
                return ConnectorBootstrap.SslServerSocket.defaultSSLSocketFactory = (SSLSocketFactory)SSLSocketFactory.getDefault();
            }
            return ConnectorBootstrap.SslServerSocket.defaultSSLSocketFactory;
        }
    }
    
    public interface DefaultValues
    {
        public static final String PORT = "0";
        public static final String CONFIG_FILE_NAME = "management.properties";
        public static final String USE_SSL = "true";
        public static final String USE_LOCAL_ONLY = "true";
        public static final String USE_REGISTRY_SSL = "false";
        public static final String USE_AUTHENTICATION = "true";
        public static final String PASSWORD_FILE_NAME = "jmxremote.password";
        public static final String ACCESS_FILE_NAME = "jmxremote.access";
        public static final String SSL_NEED_CLIENT_AUTH = "false";
    }
    
    public interface PropertyNames
    {
        public static final String PORT = "com.sun.management.jmxremote.port";
        public static final String HOST = "com.sun.management.jmxremote.host";
        public static final String RMI_PORT = "com.sun.management.jmxremote.rmi.port";
        public static final String CONFIG_FILE_NAME = "com.sun.management.config.file";
        public static final String USE_LOCAL_ONLY = "com.sun.management.jmxremote.local.only";
        public static final String USE_SSL = "com.sun.management.jmxremote.ssl";
        public static final String USE_REGISTRY_SSL = "com.sun.management.jmxremote.registry.ssl";
        public static final String USE_AUTHENTICATION = "com.sun.management.jmxremote.authenticate";
        public static final String PASSWORD_FILE_NAME = "com.sun.management.jmxremote.password.file";
        public static final String ACCESS_FILE_NAME = "com.sun.management.jmxremote.access.file";
        public static final String LOGIN_CONFIG_NAME = "com.sun.management.jmxremote.login.config";
        public static final String SSL_ENABLED_CIPHER_SUITES = "com.sun.management.jmxremote.ssl.enabled.cipher.suites";
        public static final String SSL_ENABLED_PROTOCOLS = "com.sun.management.jmxremote.ssl.enabled.protocols";
        public static final String SSL_NEED_CLIENT_AUTH = "com.sun.management.jmxremote.ssl.need.client.auth";
        public static final String SSL_CONFIG_FILE_NAME = "com.sun.management.jmxremote.ssl.config.file";
    }
}
